/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):                  by Poul-Henning Kamp
 * <j@uriah.heep.sax.de> wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.        Joerg Wunsch
 * ----------------------------------------------------------------------------
 *
 */

#include <stdio.h>

#include <avr/io.h>
#include <avr/pgmspace.h>

/*
 * Do all the startup-time peripheral initializations.
 */
void
ioinit(void)
{

#if F_CPU < 2000000UL
  UCSR1A = _BV(U2X1);             /* improve baud rate error by using 2x clk */
  UBRR1 = (F_CPU / (8 * 9600UL)) - 1; /* 9600 Bd */
#else
  UBRR1 = (F_CPU / (16 * 9600UL)) - 1; /* 9600 Bd */
#endif
  UCSR1B = _BV(TXEN1);
}

/*
 * Send character c down the UART Tx, wait until tx holding register
 * is empty.
 */
int
uart_putchar(char c, FILE *stream)
{

  if (c == '\n')
    uart_putchar('\r', stream);
  loop_until_bit_is_set(UCSR1A, UDRE1);
  UDR1 = c;
  return 0;
}

static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL,
                                         _FDEV_SETUP_WRITE);
int
main(void)
{
    ioinit();
    stdout = &mystdout;

    int i;
    for (i = 0; i < 10; i++)
        printf_P(PSTR("Hello, world, #%d!\n"), i);

    for (;;)
        ;

    return 42;
}
